/*
 * Copyright 2012-2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.spring.start.site.extension.dependency.graalvm;

import io.spring.initializr.generator.buildsystem.gradle.GradleBuild;
import io.spring.initializr.generator.version.Version;

/**
 * {@link GraalVmGradleBuildCustomizer} implementations for projects using the Kotlin DSL.
 *
 * @author Stephane Nicoll
 */
class GraalVmKotlinDslGradleBuildCustomizer extends GraalVmGradleBuildCustomizer {

	private static final String BOOT_BUILD_IMAGE_TASK = "org.springframework.boot.gradle.tasks.bundling.BootBuildImage";

	GraalVmKotlinDslGradleBuildCustomizer(Version platformVersion) {
		super(platformVersion);
	}

	@Override
	protected void customizeSpringBootPlugin(GradleBuild build) {
		build.tasks().customizeWithType(BOOT_BUILD_IMAGE_TASK, (task) -> task.invoke("buildpacks.set",
				"listOf(\"gcr.io/paketo-buildpacks/bellsoft-liberica:9.9.0-ea\", \"gcr.io/paketo-buildpacks/java-native-image\")"));
	}

}
